<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use SM\Traits\TQuery;

class ProposalModelFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getProposalModel()
    {
        $sql = "SELECT 
            ORC.NRORCAMENTOMODELO,
            ORC.NRARVOREPAI,
            ORC.NRTIPOORCAMENTO,
            ORC.NRPARCTRANSPORTADORA,
            ORC.CDREDUCONT,
            ORC.NMORCAMENTOMODELO,
            ORC.DSREFERENCIAORCA,
            ORC.IDTRANSPORTADORA,
            ORC.QTDIASVALIDADE,
            ORC.QTDIASCONTRATO,
            ORC.DSOBSERVACAOORCA,
            ORC.CDMINEMORCA,
            ORC.IDUTILIZAREVENDA,
            ORC.IDFATURACONTR,
            ORC.CDMODPGTO,
            ORC.NRGRUPOFATURA,
            ORC.IDUTILIZAHIERSERV,
            ORC.IDBLOQALTERADADOS,
            ORC.IDOBRIGAOPORTUNIDADE,
            ORC.IDUTILIZACOMP,
            ORC.NRDIASCARENCIA,
            TIPO.NMTIPOORCAMENTO,
            GRUPO.NMGRUPOFATURA,
            MODPGTO.DSMODPGTO,
            ORC.NRORG,
            ORC.IDATIVO,
            ORC.DTINCLUSAO,
            ORC.DTULTATU,
            ORC.NRORGINCLUSAO,
            ORC.CDOPERINCLUSAO,
            ORC.NRORGULTATU,
            ORC.CDOPERULTATU,
            ORC.QTPARCELASFATU,
            ORC.NRDIAVENCPARCELASFATU,
            ORC.NRSEQMODCONTRA,
            MODCONT.NMMODCONTRATO,
            ORC.IDASSELETRDIGITAL,
            ORC.IDENVIADOCCONCLUSAO,
            ORC.IDENVIADOCOPERRESPCONCASS,
            ORC.NRTIPODOCCERTSIGN,
            TIPODOC.NMTIPODOCCERTSIGN
        FROM 
            FCT_ORCAMENTOMODELO ORC
        LEFT JOIN
            FCT_TIPOORCAMENTO TIPO ON ORC.NRTIPOORCAMENTO = TIPO.NRTIPOORCAMENTO
        AND ORC.NRORG = TIPO.NRORG
        LEFT JOIN FCT_GRUPOFATURA GRUPO ON ORC.NRGRUPOFATURA = GRUPO.NRGRUPOFATURA
        AND ORC.NRORG = GRUPO.NRORG LEFT JOIN MODPGTO ON ORC.CDMODPGTO = MODPGTO.CDMODPGTO AND ORC.NRORG = MODPGTO.NRORG 
        LEFT JOIN FCT_MODELOCONTRATO MODCONT ON ORC.NRSEQMODCONTRA = MODCONT.NRSEQMODCONTRA AND ORC.NRORG = MODCONT.NRORG 
        LEFT JOIN FCT_TIPODOCCERTSIGN TIPODOC ON ORC.NRTIPODOCCERTSIGN = TIPODOC.NRTIPODOCCERTSIGN AND ORC.NRORG = TIPODOC.NRORG";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTabEscopoFaturamentoProposal($params)
    {
        $sql = "SELECT FCT_RELACESCOPOMODELO.NRRELACESCOPOMODELO,
            FCT_RELACESCOPOMODELO.NRORCAMENTOMODELO,
            FCT_RELACESCOPOMODELO.NRTIPOORCAMENTO,
            FCT_TIPOORCAMENTO.NMTIPOORCAMENTO,
            FCT_RELACESCOPOMODELO.IDVERIFICADOC,
            FCT_TIPOORCAMENTO.IDUTILIZAVARPRECO,
            FCT_RELACESCOPOMODELO.NRSEQMODDOCFCT,
            FCT_MODELODOC.DSMODELODOCORCA,
            FCT_RELACESCOPOMODELO.NRORG,
            FCT_RELACESCOPOMODELO.DTINCLUSAO,
            FCT_RELACESCOPOMODELO.NRORGINCLUSAO,
            FCT_RELACESCOPOMODELO.CDOPERINCLUSAO,
            OPINC.NMOPERADOR AS NMOPERINCLUSAO,
            FCT_RELACESCOPOMODELO.DTULTATU,
            FCT_RELACESCOPOMODELO.NRORGULTATU,
            FCT_RELACESCOPOMODELO.CDOPERULTATU,
            OPATU.NMOPERADOR AS NMOPERULTATU,
            FCT_RELACESCOPOMODELO.IDATIVO
        FROM FCT_RELACESCOPOMODELO 
        JOIN FCT_TIPOORCAMENTO ON FCT_RELACESCOPOMODELO.NRTIPOORCAMENTO = FCT_TIPOORCAMENTO.NRTIPOORCAMENTO
        AND FCT_RELACESCOPOMODELO.NRORG = FCT_TIPOORCAMENTO.NRORG
        LEFT JOIN OPERADOR OPINC ON FCT_RELACESCOPOMODELO.CDOPERINCLUSAO = OPINC.CDOPERADOR
        AND FCT_RELACESCOPOMODELO.NRORG = OPINC.NRORG
        LEFT JOIN OPERADOR OPATU ON FCT_RELACESCOPOMODELO.CDOPERULTATU = OPATU.CDOPERADOR
        AND FCT_RELACESCOPOMODELO.NRORG = OPATU.NRORG
        LEFT JOIN FCT_MODELODOC ON FCT_RELACESCOPOMODELO.NRSEQMODDOCFCT = FCT_MODELODOC.NRSEQMODDOCFCT
        AND FCT_RELACESCOPOMODELO.NRORG = FCT_MODELODOC.NRORG
        WHERE FCT_RELACESCOPOMODELO.NRORCAMENTOMODELO = :NRORCAMENTOMODELO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getEscopoFaturamentoModeloDoc($params) 
    {
        $sql = "SELECT
                    MODDOCESC.IDPERMITEDOCX,
                    MODDOCESC.NRRELACESCOPOMODELO,
                    MODDOCESC.NRSEQMODDOCFCT,
                    LISTAGG(MODDOCESC.NRORCAESCOPODOC, ', ') WITHIN GROUP(
                    ORDER BY
                        MODDOCESC.NRORCAESCOPODOC
                    ) AS NRORCAESCOPODOC,
                    MODDOCESC.NRORG,
                    MODDOC.DSMODELODOCORCA,
                    LISTAGG(MODDOCESC.IDSITUACAOORCAMENTO, ',') WITHIN GROUP(
                    ORDER BY
                        MODDOCESC.IDSITUACAOORCAMENTO
                    ) AS IDSITUACAOORCAMENTO
                FROM
                        FCT_ORCAESCOPODOC MODDOCESC
                    INNER JOIN FCT_MODELODOC MODDOC ON MODDOC.NRSEQMODDOCFCT = MODDOCESC.NRSEQMODDOCFCT
                                                    AND MODDOC.NRORG = MODDOCESC.NRORG
                                                    WHERE NRRELACESCOPOMODELO = :NRRELACESCOPOMODELO
                GROUP BY
                    MODDOCESC.IDPERMITEDOCX,
                    MODDOCESC.NRRELACESCOPOMODELO,
                    MODDOCESC.NRSEQMODDOCFCT,
                    MODDOCESC.NRORG,
                    MODDOC.DSMODELODOCORCA";
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params, [], null, null, null, array('IDSITUACAOORCAMENTO'));
    }
    /**
     * Deleta o modelo de documento ligado a um escopo
    **/
    public function excluirEscFatModeloDocumento(array $params) {
        $sql = "DELETE FROM FCT_ORCAESCOPODOC
                WHERE NRRELACESCOPOMODELO   = :NRRELACESCOPOMODELO
                    AND NRSEQMODDOCFCT      = :NRSEQMODDOCFCT
                    AND NRORG               = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    /**
     * Deleta o documento de determinada situação ligada a um escopo
    **/
    public function excluirDocumentoSituacao($idSituacao, array $params) {
        $sql = "DELETE FROM FCT_ORCAESCOPODOC
                WHERE IDSITUACAOORCAMENTO IN('$idSituacao')
                    AND NRRELACESCOPOMODELO = :NRRELACESCOPOMODELO
                    AND NRORG               = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Edita as informações de um documento ligado a um escopo
    **/
    public function editarDocumentoSituacao($idSituacao, array $params){
        $sql = "UPDATE FCT_ORCAESCOPODOC
                SET IDPERMITEDOCX  = :IDPERMITEDOCX,
                    CDOPERULTATU   = :CDOPERADOR,
                    NRORGULTATU    = :NRORG,
                    DTULTATU       = SYSDATE
                WHERE NRRELACESCOPOMODELO = :NRRELACESCOPOMODELO
                    AND NRORG             = :NRORG
                    AND NRSEQMODDOCFCT    = :NRSEQMODDOCFCT";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Insere um modelo de docuemnto para uma determinada situação  
    **/
    public function inserirDocumentoSituacao(array $params) {
        $sql = "INSERT INTO FCT_ORCAESCOPODOC (
                    NRORCAESCOPODOC,     NRRELACESCOPOMODELO, NRSEQMODDOCFCT,
                    IDSITUACAOORCAMENTO, NRORG,               IDATIVO,
                    DTINCLUSAO,          DTULTATU,            NRORGINCLUSAO,
                    CDOPERINCLUSAO,      NRORGULTATU,         CDOPERULTATU,
                    IDPERMITEDOCX
                ) VALUES (
                    :NRORCAESCOPODOC,     :NRRELACESCOPOMODELO, :NRSEQMODDOCFCT,
                    :IDSITUACAOORCAMENTO, :NRORG,               :IDATIVO,
                    SYSDATE,               NULL,                :NRORG,
                    :CDOPERADOR,           NULL,                 NULL,
                    :IDPERMITEDOCX
                )";
        $std = $this->connection->prepare($sql);
        $std->execute($params);    
    }
    /**
     * Retorna os modelos de documento dos escopo por situação
    **/
    public function getModeloDocumentoSituacao($idSituacao, array $params) {
        $sql = "SELECT LISTAGG(IDSITUACAOORCAMENTO, ', ') WITHIN GROUP (ORDER BY IDSITUACAOORCAMENTO) AS IDSITUACAOORCAMENTO 
                FROM FCT_ORCAESCOPODOC ORCDOC
                JOIN FCT_MODELODOC DOC
                    ON ORCDOC.NRSEQMODDOCFCT = DOC.NRSEQMODDOCFCT
                    AND ORCDOC.NRORG         = DOC.NRORG
                WHERE ORCDOC.NRORG                 = :NRORG
                    AND ORCDOC.NRRELACESCOPOMODELO = :NRRELACESCOPOMODELO
                    AND ORCDOC.IDSITUACAOORCAMENTO IN('$idSituacao') 
                    AND DOC.IDDOCRENEGOCIACAO = (SELECT IDDOCRENEGOCIACAO 
                                               FROM FCT_MODELODOC
                                               WHERE NRSEQMODDOCFCT = :NRSEQMODDOCFCT)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getTabComposicoes($params)
    {
        $sql = "SELECT FCT_RELORCAMODCOMPOSICAO.NRRELORCAMODCOMP,
            FCT_RELORCAMODCOMPOSICAO.NRORCAMENTOMODELO,
            FCT_RELORCAMODCOMPOSICAO.NRCOMPSERVICO,
            FCT_COMPSERVICO.NMCOMPSERVICO,
            FCT_COMPSERVICO.NRCOMPSERVICOPAI,
            COMP_PAI.NMCOMPSERVICO AS NMCOMPSERVICOPAI,
            FCT_RELORCAMODCOMPOSICAO.NRORG,
            FCT_RELORCAMODCOMPOSICAO.DTINCLUSAO,
            FCT_RELORCAMODCOMPOSICAO.NRORGINCLUSAO,
            FCT_RELORCAMODCOMPOSICAO.CDOPERINCLUSAO,
            OPINC.NMOPERADOR AS NMOPERINCLUSAO,
            FCT_RELORCAMODCOMPOSICAO.DTULTATU,
            FCT_RELORCAMODCOMPOSICAO.NRORGULTATU,
            FCT_RELORCAMODCOMPOSICAO.CDOPERULTATU,
            OPATU.NMOPERADOR AS NMOPERULTATU,
            FCT_RELORCAMODCOMPOSICAO.IDATIVO
        FROM
            FCT_RELORCAMODCOMPOSICAO
        INNER JOIN FCT_COMPSERVICO ON FCT_RELORCAMODCOMPOSICAO.NRCOMPSERVICO = FCT_COMPSERVICO.NRCOMPSERVICO
        AND FCT_RELORCAMODCOMPOSICAO.NRORG = FCT_COMPSERVICO.NRORG
        LEFT JOIN FCT_COMPSERVICO COMP_PAI ON FCT_COMPSERVICO.NRCOMPSERVICOPAI = COMP_PAI.NRCOMPSERVICO
        AND FCT_COMPSERVICO.NRORG = COMP_PAI.NRORG
        LEFT JOIN OPERADOR OPINC ON FCT_RELORCAMODCOMPOSICAO.CDOPERINCLUSAO = OPINC.CDOPERADOR
        AND FCT_RELORCAMODCOMPOSICAO.NRORG = OPINC.NRORG
        LEFT JOIN OPERADOR OPATU ON FCT_RELORCAMODCOMPOSICAO.CDOPERULTATU = OPATU.CDOPERADOR
        AND FCT_RELORCAMODCOMPOSICAO.NRORG = OPATU.NRORG
        WHERE FCT_RELORCAMODCOMPOSICAO.NRORCAMENTOMODELO = :NRORCAMENTOMODELO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getTabItens($params)
    {
        $sql = "SELECT
            ITEM.NRITEMORCAMODELO,
            ITEM.NRORCAMENTOMODELO,
            ITEM.NRITEMORCAMODELOPAI,
            ITEM.CDSERVORCA,
            ITEM.CDPRODUTO,
            ITEM.SGUNIDADE,
            ITEM.SGUNIDADEADICIONAL,
            ITEM.CDPRECBASESERV,
            ITEM.NRGRUPOFATURA,
            ITEM.NRPARCTRANSPORTADORA,
            ITEM.NRPARCFORNECEDOR,
            ITEM.QTITEMORCAMENTO,
            ITEM.CDTPMEDIDASERV,
            ITEM.QTMEDIDAORCA,
            ITEM.IDFATURAORCA,
            ITEM.CDMODPGTO,
            ITEM.VRUNITITEMORCA,
            ITEM.VRTOTALITEMORCA,
            DECODE(ITEM.IDITEMESPELHO, 'SIM', 'Item Espelho', ITEM.NMITEMORCAMENTO) AS NMITEMORCAMENTO,
            ITEM.NMREDUITEMORCA,
            ITEM.NMMNEMOITEMORCA,
            ITEM.DSQTITEMORCA,
            ITEM.DSQTITEMORCAADIC,
            ITEM.QTITEMORCAADIC,
            ITEM.QTDIASINICFAT,
            ITEM.IDITEMOBRIGORCA,
            ITEM.IDMULTMEDVRTOT,
            ITEM.IDBLOQALTORC,
            ITEM.IDADICITEMORC,
            ITEM.IDADICCOBRITEMORC,
            ITEM.IDSOMASUBITENS,
            ITEM.IDSOMANOITEMPAI,
            ITEM.IDSOMANOTOTALORCA,
            ITEM.DSAGRUPAITEMORCA,
            ITEM.IDIMPRESSAOORCA,
            ITEM.IDTRANSPORTADORA,
            ITEM.TXFORMULAITORCA,
            ITEM.QTDIASCONTRATO,
            ITEM.NRORG,
            ITEM.IDATIVO,
            ITEM.DTINCLUSAO,
            ITEM.DTULTATU,
            ITEM.NRORGINCLUSAO,
            ITEM.CDOPERINCLUSAO,
            ITEM.NRORGULTATU,
            ITEM.CDOPERULTATU,
            ITEM.IDITEMESPELHO,
            ITEM.NRTIPOHIERARQSERV,
            SERV.NMSERVORCA,
            MEDIDA.DSTPMEDIDASERV,
            MODPGTO.DSMODPGTO,
            FAT.NMGRUPOFATURA,
            PRODUTO.NMPRODUTO,
            DECODE(ITEM.CDPRODUTO, NULL, 'S', 'P')                                  AS TIPO,
            NVL(ITEM.CDPRODUTO, ITEM.CDSERVORCA)                                    AS CODIGO,
            ITEM.QTPARCELASFATU,
            ITEM.NRDIAVENCPARCELASFATU
        FROM
            FCT_ITEMORCAMODELO ITEM
            LEFT JOIN FCT_SERVICOORCA    SERV ON ITEM.CDSERVORCA = SERV.CDSERVORCA
                                            AND ITEM.NRORG = SERV.NRORG
            LEFT JOIN FCT_TIPOMEDIDASERV MEDIDA ON ITEM.CDTPMEDIDASERV = MEDIDA.CDTPMEDIDASERV
                                                AND ITEM.NRORG = MEDIDA.NRORG
            LEFT JOIN MODPGTO ON ITEM.CDMODPGTO = MODPGTO.CDMODPGTO
                                AND ITEM.NRORG = MODPGTO.NRORG
            LEFT JOIN FCT_GRUPOFATURA    FAT ON ITEM.NRGRUPOFATURA = FAT.NRGRUPOFATURA
                                            AND ITEM.NRORG = FAT.NRORG
            LEFT JOIN PRODUTO ON ITEM.CDPRODUTO = PRODUTO.CDPRODUTO
            AND ITEM.NRORG = PRODUTO.NRORG
        WHERE ITEM.NRORCAMENTOMODELO = :NRORCAMENTOMODELO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getTabPerfisAcessoProposal($params)
    {
        $sql = "SELECT
            REL.NRRELORCAMODPERFACESS,
            REL.NRORCAMENTOMODELO,
            REL.NRPERFILACESSO,
            REL.NRORG,
            REL.IDATIVO,
            REL.DTINCLUSAO,
            REL.NRORGINCLUSAO,
            REL.CDOPERINCLUSAO,
            REL.DTULTATU,
            REL.NRORGULTATU,
            REL.CDOPERULTATU,
            PER.NMPERFILACESSO
        FROM
                FCT_RELORCAMODPERFACESS REL
        JOIN PERFILACESSO PER ON PER.NRPERFILACESSO = REL.NRPERFILACESSO
        WHERE REL.NRORCAMENTOMODELO = :NRORCAMENTOMODELO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getTabSignatariosPadroesProposal($params)
    {
        $sql = "SELECT
            FCT_SIGNATPADRMODORCA.NRSIGNATPADRORCA,
            FCT_SIGNATPADRMODORCA.NRORCAMENTOMODELO,
            FCT_SIGNATPADRMODORCA.NRPARCNEGOCIO,
            PARCNEGOCIO.NMPRINCIPALPARC,
            PARCNEGOCIO.NRINSCRICAOPARC,
            FCT_SIGNATPADRMODORCA.IDTIPOASSINATURA,
            FCT_SIGNATPADRMODORCA.IDTESTEMUNHA,
            FCT_SIGNATPADRMODORCA.NRORDENACAO,
            FCT_SIGNATPADRMODORCA.IDATIVO,
            FCT_SIGNATPADRMODORCA.NRORG,
            FCT_SIGNATPADRMODORCA.DTINCLUSAO,
            FCT_SIGNATPADRMODORCA.NRORGINCLUSAO,
            FCT_SIGNATPADRMODORCA.CDOPERINCLUSAO,
            FCT_SIGNATPADRMODORCA.DTULTATU,
            FCT_SIGNATPADRMODORCA.NRORGULTATU,
            FCT_SIGNATPADRMODORCA.CDOPERULTATU,
            FCT_SIGNATPADRMODORCA.IDTPSIGNATARIO,
            FCT_SIGNATPADRMODORCA.DSEMAILNOTSIGNENV,
            FCT_SIGNATPADRMODORCA.DSEMAILNOTIFOUTROS,
            (
                SELECT
                    RTRIM(XMLAGG(XMLELEMENT(E, CDOPERADOR, ',').EXTRACT('//text()')
                        ORDER BY
                            NRRELOPERNOTIFASSELE
                    ).GETCLOBVAL(), ',') AS NRRELOPERNOTIFASSELE
                FROM
                    FCT_RELACOPERNOTIFASSELE
                WHERE
                        NRSIGNATPADRORCA = FCT_SIGNATPADRMODORCA.NRSIGNATPADRORCA
                    AND NRORG = FCT_SIGNATPADRMODORCA.NRORG
            ) AS CDOPERADORES
        FROM
            FCT_SIGNATPADRMODORCA
            LEFT JOIN PARCNEGOCIO ON FCT_SIGNATPADRMODORCA.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO
            AND FCT_SIGNATPADRMODORCA.NRORG = PARCNEGOCIO.NRORG
            JOIN FCT_ORCAMENTOMODELO ON FCT_SIGNATPADRMODORCA.NRORCAMENTOMODELO = FCT_ORCAMENTOMODELO.NRORCAMENTOMODELO
            AND FCT_SIGNATPADRMODORCA.NRORG = FCT_ORCAMENTOMODELO.NRORG
        WHERE FCT_SIGNATPADRMODORCA.NRORCAMENTOMODELO = :NRORCAMENTOMODELO
        ORDER BY
            FCT_SIGNATPADRMODORCA.NRORDENACAO,
            PARCNEGOCIO.NMPRINCIPALPARC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getServicoProduto()
    {
        $sql = "SELECT
            CODIGO,
            NOME,
            TIPO,
            NRORG
        FROM
        (SELECT
                TO_CHAR(P.CDPRODUTO) AS CODIGO,
                P.NMPRODUTO          AS NOME,
                'V'                  AS TIPO,
                P.NRORG
            FROM
                PRODUTO P
            WHERE
                    P.CDNVPRODUTO = (
                        SELECT
                            MAX(CDNVPRODUTO)
                        FROM
                            PRODUTO
                    )
                AND P.IDATIVO = 'S'
            UNION ALL
            SELECT
                TO_CHAR(S.CDSERVORCA) AS CODIGO,
                S.NMSERVORCA          AS NOME,
                'S'                   AS TIPO,
                S.NRORG
            FROM
                FCT_SERVICOORCA S
            WHERE
                    S.IDTIPO = 'S'
                AND S.IDATIVO = 'S'
        ) FCT_SERVICOORCA
        ORDER BY
            TIPO,
            CODIGO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTipoMedidaProposal()
    {
        $sql = "SELECT
            FCT_TIPOMEDIDASERV.CDTPMEDIDASERV,
            FCT_TIPOMEDIDASERV.DSTPMEDIDASERV,
            FCT_TIPOMEDIDASERV.NRORG,
            FCT_TIPOMEDIDASERV.DTINCLUSAO,
            FCT_TIPOMEDIDASERV.NRORGINCLUSAO,
            FCT_TIPOMEDIDASERV.CDOPERINCLUSAO,
            OPINC.NMOPERADOR AS NMOPERINCLUSAO,
            FCT_TIPOMEDIDASERV.DTULTATU,
            FCT_TIPOMEDIDASERV.NRORGULTATU,
            FCT_TIPOMEDIDASERV.CDOPERULTATU,
            OPATU.NMOPERADOR AS NMOPERULTATU,
            FCT_TIPOMEDIDASERV.IDATIVO,
            FCT_TIPOMEDIDASERV.IDTPMEDIDASERV
        FROM
            FCT_TIPOMEDIDASERV
        LEFT JOIN OPERADOR OPINC ON FCT_TIPOMEDIDASERV.CDOPERINCLUSAO = OPINC.CDOPERADOR
        AND FCT_TIPOMEDIDASERV.NRORG = OPINC.NRORG
        LEFT JOIN OPERADOR OPATU ON FCT_TIPOMEDIDASERV.CDOPERULTATU = OPATU.CDOPERADOR
        AND FCT_TIPOMEDIDASERV.NRORG = OPATU.NRORG";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    /**
     * valida se o Token do Certsign está preenchido
    **/
    public function validaTokenCertsign(array $params) {
        $sql = "SELECT DSTOKENCERTSIGN 
                  FROM FCT_PARAMETROGERAL
                 WHERE NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * Busca os signatários padrões de modelo de proposta da parametrização SM 
    **/
    public function buscaSignatariosPadroes(array $params) {
        $sql = "SELECT NRSIGNATARIOSPADR, NRPARCNEGOCIO, IDTIPOASSINATURA, IDTESTEMUNHA, NRORDENACAO, 
                       IDINSERCAOAUTO, IDTPSIGNATARIO, DSEMAILNOTSIGNENV, DSEMAILNOTIFOUTROS, NRORG 
                  FROM FCT_SIGNATARIOSPADR
                 WHERE IDINSERCAOAUTO IN ('P','A')
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * A função "validaSignatCadastrados" realiza a validação dos signatários padrões para evitar a inclusão repetida de um parceiro.
     */
    public function validaSignatCadastrados(array $params) {
        
        $sql = "SELECT COUNT (*) AS VALIDA
                  FROM FCT_SIGNATPADRMODORCA
                 WHERE NRORCAMENTOMODELO = :NRORCAMENTOMODELO 
                   AND (
                       (IDTPSIGNATARIO = 'SE' AND NRPARCNEGOCIO = :NRPARCNEGOCIO) OR
                       (IDTPSIGNATARIO <>'SE' AND IDTPSIGNATARIO = :IDTPSIGNATARIO)
                    )
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $retorno = $std->fetchAll();
        return $retorno[0]["VALIDA"];
    }

    /**
     * insere os signatários padrões de modelo de proposta que foram buscados na parametrização SM 
    **/
    public function insereSignatariosPadroes(array $params) {
        
        $sql = "INSERT INTO FCT_SIGNATPADRMODORCA (NRSIGNATPADRORCA, NRORCAMENTOMODELO, NRPARCNEGOCIO, IDTIPOASSINATURA, IDTESTEMUNHA,
                                                   NRORDENACAO,  IDTPSIGNATARIO, DSEMAILNOTSIGNENV, DSEMAILNOTIFOUTROS, NRORG, DTINCLUSAO, 
                                                   CDOPERINCLUSAO, NRORGINCLUSAO, IDATIVO)
                            VALUES (:NRSIGNATPADRORCA, :NRORCAMENTOMODELO, :NRPARCNEGOCIO, :IDTIPOASSINATURA, :IDTESTEMUNHA, 
                                    :NRORDENACAO, :IDTPSIGNATARIO, :DSEMAILNOTSIGNENV, :DSEMAILNOTIFOUTROS, :NRORG, SYSDATE,
                                    :OPERADOR, :NRORG, 'S')";
                             
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    /**
     * Busca os operadores padrões para notificação de modelo de proposta da parametrização SM 
    **/
    public function buscaOperadoresNotif(array $params) {
        $sql = "SELECT CDOPERADOR 
                  FROM FCT_RELACOPERNOTIFASSELE
                 WHERE NRSIGNATARIOSPADR = :NRSIGNATARIOSPADR
                   AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * insere os operadores de notificação de assinatura
    **/
    public function insereOperadoresNotifica (array $params) {
        
        $sql = "INSERT INTO FCT_RELACOPERNOTIFASSELE
                    (NRRELOPERNOTIFASSELE, NRSIGNATPADRORCA, CDOPERADOR, NRORG, IDATIVO,            
                	 DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES 
                    (:NRRELOPERNOTIFASSELE, :NRSIGNATPADRORCA, :CDOPERADOR, :NRORG, 'S',             
                	 SYSDATE, :CDOPERAINCLUSAO, :NRORG)";
                             
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function saveModalAddProposalModel($params)
    {
        $sql = "INSERT INTO FCT_ORCAMENTOMODELO
                    (NRORCAMENTOMODELO, IDATIVO, IDBLOQALTERADADOS, IDFATURACONTR, NMORCAMENTOMODELO, 
                    QTDIASVALIDADE, QTDIASCONTRATO, IDUTILIZACOMP, NRDIASCARENCIA, IDOBRIGAOPORTUNIDADE, 
                    NRSEQMODCONTRA, CDMINEMORCA, IDUTILIZAREVENDA, DSREFERENCIAORCA, DSOBSERVACAOORCA, 
                    IDASSELETRDIGITAL, IDENVIADOCCONCLUSAO, NRTIPODOCCERTSIGN, IDENVIADOCOPERRESPCONCASS, 
                    NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, DTINCLUSAO)  
                VALUES 
                    (:NRORCAMENTOMODELO, :IDATIVO, :IDBLOQALTERADADOS, :IDFATURACONTR, :NMORCAMENTOMODELO,
                    :QTDIASVALIDADE, :QTDIASCONTRATO, :IDUTILIZACOMP, :NRDIASCARENCIA, :IDOBRIGAOPORTUNIDADE, 
                    :NRSEQMODCONTRA, :CDMINEMORCA, :IDUTILIZAREVENDA, :DSREFERENCIAORCA, :DSOBSERVACAOORCA, 
                    :IDASSELETRDIGITAL, :IDENVIADOCCONCLUSAO, :NRTIPODOCCERTSIGN, :IDENVIADOCOPERRESPCONCASS,
                    :NRORG, :NRORG, :CDOPERINCLUSAO, SYSDATE)";
         
        $this->connection->executeStatement($sql, $params);
    }

    public function editProposalModelViewItens($params)
    {
        $sql = "UPDATE FCT_ORCAMENTOMODELO
                   SET 
                        IDATIVO                     = :IDATIVO,
                        IDBLOQALTERADADOS           = :IDBLOQALTERADADOS,
                        NMORCAMENTOMODELO           = :NMORCAMENTOMODELO,
                        QTDIASVALIDADE              = :QTDIASVALIDADE,
                        QTDIASCONTRATO              = :QTDIASCONTRATO,
                        IDUTILIZACOMP               = :IDUTILIZACOMP,
                        NRDIASCARENCIA              = :NRDIASCARENCIA,
                        IDOBRIGAOPORTUNIDADE        = :IDOBRIGAOPORTUNIDADE,
                        CDMINEMORCA                 = :CDMINEMORCA,
                        DSREFERENCIAORCA            = :DSREFERENCIAORCA,
                        DSOBSERVACAOORCA            = :DSOBSERVACAOORCA,
                        IDASSELETRDIGITAL           = :IDASSELETRDIGITAL,
                        IDENVIADOCCONCLUSAO         = :IDENVIADOCCONCLUSAO,
                        NRTIPODOCCERTSIGN           = :NRTIPODOCCERTSIGN,
                        IDENVIADOCOPERRESPCONCASS   = :IDENVIADOCOPERRESPCONCASS,
                        NRSEQMODCONTRA              = :NRSEQMODCONTRA,
                        NRORG                       = :NRORG,
                        CDOPERULTATU                = :CDOPERULTATU,
                        DTULTATU                    = SYSDATE
                WHERE NRORCAMENTOMODELO             = :NRORCAMENTOMODELO";

        $this->connection->executeStatement($sql, $params);
    }

    public function editProposalModel($params)
    {
        $sql = "UPDATE FCT_ORCAMENTOMODELO
                   SET 
                        IDATIVO                     = :IDATIVO,
                        IDBLOQALTERADADOS           = :IDBLOQALTERADADOS,
                        NMORCAMENTOMODELO           = :NMORCAMENTOMODELO,
                        QTDIASVALIDADE              = :QTDIASVALIDADE,
                        QTDIASCONTRATO              = :QTDIASCONTRATO,
                        IDUTILIZACOMP               = :IDUTILIZACOMP,
                        NRDIASCARENCIA              = :NRDIASCARENCIA,
                        IDOBRIGAOPORTUNIDADE        = :IDOBRIGAOPORTUNIDADE,
                        CDMINEMORCA                 = :CDMINEMORCA,
                        IDUTILIZAREVENDA            = :IDUTILIZAREVENDA,
                        DSREFERENCIAORCA            = :DSREFERENCIAORCA,
                        DSOBSERVACAOORCA            = :DSOBSERVACAOORCA,
                        IDASSELETRDIGITAL           = :IDASSELETRDIGITAL,
                        IDENVIADOCCONCLUSAO         = :IDENVIADOCCONCLUSAO,
                        NRTIPODOCCERTSIGN           = :NRTIPODOCCERTSIGN,
                        IDENVIADOCOPERRESPCONCASS   = :IDENVIADOCOPERRESPCONCASS,
                        NRSEQMODCONTRA              = :NRSEQMODCONTRA,
                        NRORG                       = :NRORG,
                        CDOPERULTATU                = :CDOPERULTATU,
                        DTULTATU                    = SYSDATE
                WHERE NRORCAMENTOMODELO             = :NRORCAMENTOMODELO";

        $this->connection->executeStatement($sql, $params);
    }

    public function deleteProposalModel($parametros)
    {
        $sql = "DELETE FROM FCT_MODELOCONTRATO
                WHERE NRSEQMODCONTRA IN (".$parametros.")";
        
        $this->connection->executeStatement($sql);
    }

    public function deleteProposalModelTabEscopoFaturamento($parametros)
    {
        $sql = "DELETE FROM FCT_RELACESCOPOMODELO
                WHERE NRRELACESCOPOMODELO IN (".$parametros.")";
        
        $this->connection->executeStatement($sql);
    }

    public function deleteProposalModelTabComposicoes($parametros)
    {
        $sql = "DELETE FROM FCT_RELORCAMODCOMPOSICAO
                WHERE NRRELORCAMODCOMP IN (".$parametros.")";
        
        $this->connection->executeStatement($sql);
    }

    public function deleteProposalModelTabPerfisAcesso($parametros)
    {
        $sql = "DELETE FROM FCT_RELORCAMODPERFACESS
                WHERE NRRELORCAMODPERFACESS IN (".$parametros.")";
        
        $this->connection->executeStatement($sql);
    }

    public function deleteProposalModelTabSignatariosPadroes($parametros)
    {
        $sql = "DELETE FROM FCT_RELACOPERNOTIFASSELE
                WHERE NRSIGNATPADRCONT IN (".$parametros.")";
        $this->connection->executeStatement($sql);

        $sql = "DELETE FROM FCT_SIGNATPADRMODCONT
                WHERE NRSIGNATPADRCONT IN (".$parametros.")";
        $this->connection->executeStatement($sql);
    }

    public function getEscopoDeFaturamentoProposal()
    {
        $sql = "SELECT
            ESCMOD.NRESCOPOMORCONT,
            ESCMOD.NRSEQMODCONTRA,
            ESCMOD.NRTIPOORCAMENTO,
            ESCMOD.NRORG,
            ESCMOD.IDATIVO,
            ESCMOD.DTINCLUSAO,
            ESCMOD.DTULTATU,
            ESCMOD.NRORGINCLUSAO,
            ESCMOD.CDOPERINCLUSAO,
            ESCMOD.NRORGULTATU,
            ESCMOD.CDOPERULTATU,
            ESCOPO.NMTIPOORCAMENTO
        FROM
                FCT_RELACESCOPOMODCONT ESCMOD
            JOIN FCT_TIPOORCAMENTO ESCOPO ON ESCMOD.NRTIPOORCAMENTO = ESCOPO.NRTIPOORCAMENTO
            AND ESCMOD.NRORG = ESCOPO.NRORG";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getModeloContratoProposal()
    {
        $sql = "SELECT
            NRSEQMODCONTRA,
            NMMODCONTRATO
        FROM FCT_MODELOCONTRATO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    /**
     * A função "validaEscopoProposta" verifica se o escopo 
     * que está sendo relacionado já está relacionado ao modelo.
     */
    
     public function validaEscopoProposta($params){
        
        $queryValidaEscopoProp = "SELECT COUNT(*) AS VALIDAESCOPO
                                    FROM FCT_RELACESCOPOMODELO
                                   WHERE NRTIPOORCAMENTO IN (".$params['NRTIPOORCAMENTO'].")
                                     AND NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                                     AND NRORG = :NRORG";

        unset($params['NRTIPOORCAMENTO']);
        $std = $this->connection->prepare($queryValidaEscopoProp);
        $std->execute($params);
        $returnValidaEscopoProp = $std->fetchAll();
        return $returnValidaEscopoProp;
    }

    public function saveModalAddProposalModelTabEscopoFaturamento($params)
    {
        $sql = "INSERT INTO FCT_RELACESCOPOMODELO 
                        (NRORCAMENTOMODELO,
                        NRTIPOORCAMENTO,
                        IDVERIFICADOC,
                        NRRELACESCOPOMODELO,
                        IDATIVO,
                        NRORG,
                        CDOPERINCLUSAO,
                        DTINCLUSAO)

                VALUES (:NRORCAMENTOMODELO,    
                        :NRTIPOORCAMENTO,
                        :IDVERIFICADOC,
                        :NRRELACESCOPOMODELO,
                        :IDATIVO,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $params);
    }

    public function saveModalAddProposalModelTabComposicoes($params)
    {
        $sql = "INSERT INTO FCT_RELORCAMODCOMPOSICAO 
                       (NRRELORCAMODCOMP,
                        NRORCAMENTOMODELO,
                        NRCOMPSERVICO,
                        IDATIVO,
                        NRORG,
                        CDOPERINCLUSAO,
                        CDOPERULTATU,
                        DTINCLUSAO)

                VALUES (:NRRELORCAMODCOMP,
                        :NRORCAMENTOMODELO,
                        :NRCOMPSERVICO,
                        :IDATIVO,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :CDOPERULTATU,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $params);
    }

    public function saveModalAddProposalModelTabPerfisAcesso($params)
    {
        $sql = "INSERT INTO FCT_RELORCAMODPERFACESS 
                       (NRORCAMENTOMODELO,
                        IDATIVO,
                        NRPERFILACESSO,
                        NRORG,
                        CDOPERINCLUSAO,
                        NRRELORCAMODPERFACESS,
                        DTINCLUSAO)

                VALUES (:NRORCAMENTOMODELO,
                        :IDATIVO,
                        :NRPERFILACESSO,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :NRRELORCAMODPERFACESS,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $params);
    }

    public function saveModalAddProposalModelTabSignatariosPadroes($params)
    {
        $sql = "INSERT INTO FCT_SIGNATPADRMODCONT 
                       (NRSIGNATPADRCONT,
                        NRSEQMODCONTRA,
                        IDTPSIGNATARIO,
                        NRPARCNEGOCIO,
                        IDTIPOASSINATURA,
                        IDTESTEMUNHA,
                        NRORDENACAO,
                        DSEMAILNOTSIGNENV,
                        DSEMAILNOTIFOUTROS,
                        NRORG,
                        CDOPERINCLUSAO,
                        IDATIVO,
                        DTINCLUSAO)

                VALUES (:NRSIGNATPADRCONT,
                        :NRSEQMODCONTRA,
                        :IDTPSIGNATARIO,
                        :NRPARCNEGOCIO,
                        :IDTIPOASSINATURA,
                        :IDTESTEMUNHA,
                        :NRORDENACAO,
                        :DSEMAILNOTSIGNENV,
                        :DSEMAILNOTIFOUTROS,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :IDATIVO,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $params);
        // var_dump($params);
        // die;
    }

    public function saveOperadoresSigPadrao($params)
    {
        $sql = "INSERT INTO FCT_RELACOPERNOTIFASSELE 
                       (NRRELOPERNOTIFASSELE,
                        NRSIGNATPADRCONT,
                        CDOPERADOR,
                        NRORG,
                        CDOPERINCLUSAO,
                        IDATIVO,
                        DTINCLUSAO)

                VALUES (:NRRELOPERNOTIFASSELE,
                        :NRSIGNATPADRCONT,
                        :CDOPERADOR,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :IDATIVO,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $params);
        // var_dump('saveOperadoresSigPadrao');
        // die;
    }

    public function getComposicoes()
    {
        $sql = "SELECT FCT_RELORCAMODCOMPOSICAO.NRRELORCAMODCOMP,
            FCT_RELORCAMODCOMPOSICAO.NRORCAMENTOMODELO,
            FCT_RELORCAMODCOMPOSICAO.NRCOMPSERVICO,
            FCT_COMPSERVICO.NMCOMPSERVICO,
            FCT_COMPSERVICO.NRCOMPSERVICOPAI,
            COMP_PAI.NMCOMPSERVICO AS NMCOMPSERVICOPAI,
            FCT_RELORCAMODCOMPOSICAO.NRORG,
            FCT_RELORCAMODCOMPOSICAO.DTINCLUSAO,
            FCT_RELORCAMODCOMPOSICAO.NRORGINCLUSAO,
            FCT_RELORCAMODCOMPOSICAO.CDOPERINCLUSAO,
            OPINC.NMOPERADOR AS NMOPERINCLUSAO,
            FCT_RELORCAMODCOMPOSICAO.DTULTATU,
            FCT_RELORCAMODCOMPOSICAO.NRORGULTATU,
            FCT_RELORCAMODCOMPOSICAO.CDOPERULTATU,
            OPATU.NMOPERADOR AS NMOPERULTATU,
            FCT_RELORCAMODCOMPOSICAO.IDATIVO
        FROM
            FCT_RELORCAMODCOMPOSICAO
        INNER JOIN FCT_COMPSERVICO ON FCT_RELORCAMODCOMPOSICAO.NRCOMPSERVICO = FCT_COMPSERVICO.NRCOMPSERVICO
        AND FCT_RELORCAMODCOMPOSICAO.NRORG = FCT_COMPSERVICO.NRORG
        LEFT JOIN FCT_COMPSERVICO COMP_PAI ON FCT_COMPSERVICO.NRCOMPSERVICOPAI = COMP_PAI.NRCOMPSERVICO
        AND FCT_COMPSERVICO.NRORG = COMP_PAI.NRORG
        LEFT JOIN OPERADOR OPINC ON FCT_RELORCAMODCOMPOSICAO.CDOPERINCLUSAO = OPINC.CDOPERADOR
        AND FCT_RELORCAMODCOMPOSICAO.NRORG = OPINC.NRORG
        LEFT JOIN OPERADOR OPATU ON FCT_RELORCAMODCOMPOSICAO.CDOPERULTATU = OPATU.CDOPERADOR
        AND FCT_RELORCAMODCOMPOSICAO.NRORG = OPATU.NRORG";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTipoDocCertsign()
    {
        $sql = "SELECT
            NRTIPODOCCERTSIGN,
            NMTIPODOCCERTSIGN
        FROM FCT_TIPODOCCERTSIGN";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getPerfisDeAcessoProposal()
    {
        $sql = "SELECT
            NRPERFILACESSO,
            NMPERFILACESSO
        FROM PERFILACESSO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getSignatariosProposalModel()
    {
        $sql = "SELECT
                    PARCNEGOCIO.NRPARCNEGOCIO,
                    PARCNEGOCIO.NMPRINCIPALPARC,
                    PARCNEGOCIO.NRINSCRICAOPARC,
                    PARCNEGOCIO.NMSECUNDARIPARC,
                    OPERADOR.CDOPERADOR,
                    PARCNEGOCIO.NRORG
                FROM
                        PARCNEGOCIO
                JOIN PARCERIA ON PARCNEGOCIO.NRPARCNEGOCIO = PARCERIA.NRPARCNEGOCIO
                    AND PARCNEGOCIO.NRORG = PARCERIA.NRORG
                    AND PARCNEGOCIO.IDATIVO = PARCERIA.IDATIVO
                    AND PARCERIA.CDTIPOPARCEIRO = 'OPERADOR'
                    AND PARCERIA.IDATIVO = 'S'
                JOIN OPERADOR ON PARCNEGOCIO.NRPARCNEGOCIO = OPERADOR.NRPARCNEGOCIO
                    AND PARCNEGOCIO.NRORG = OPERADOR.NRORG
                    AND OPERADOR.IDOPERATIVO = 'A'
                WHERE PARCNEGOCIO.CDTIPOINSCRICAO = 'CPF'
                    AND PARCNEGOCIO.IDATIVO = 'S'
                ORDER BY PARCNEGOCIO.NMPRINCIPALPARC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getOperadoresProposalModel()
    {
        $sql = "SELECT
                    CDOPERADOR,
                    NMOPERADOR
                FROM OPERADOR
                ORDER BY NMOPERADOR";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getCpfSignatario($params)
    {
        $sql = "SELECT
                    NRINSCRICAOPARC
                FROM PARCNEGOCIO
                WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }
}
