<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use SM\Traits\TQuery;

class Sm00006000ElementoFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getElementos($fetchOnDemand, $fetchOnDemandPrimeiraLinha, $filtroIdAtivo, $filtroNome, $filtroTipoMedidaVariavel, $filtroRelacionamentoServico, $filtroFormacaoPreco, $filtroCentroCusto, $idTipoElemento)
    {
        $sql = "SELECT
                ELEMENTO.DSEXPRESSAOCALC,
                ELEMENTO.IDTIPOCALCULO,
                ELEMENTO.VRPERCCALCULO,
                ELEMENTO.IDALTERAFORMPREC,
                ELEMENTO.CDSERVORCA,
                ELEMENTO.CDFILIAL,
                ELEMENTO.IDITEMCALCULO,
                ELEMENTO.NMSERVORCA,
                ELEMENTO.DSSERVORCA,
                ELEMENTO.CDPRODFATSERV,
                ELEMENTO.IDATIVO,
                ELEMENTO.IDTIPO,
                ELEMENTO.IDELEMENTO,
                ELEMENTO.IDTIPOELEMENTO,
                ELEMENTO.IDELEMENTOPAI,
                PAI.NMSERVORCA                                                                                       AS NMSERVORCAPAI,
                ELEMENTO.NRORG,
                ELEMENTO.DTINCLUSAO,
                ELEMENTO.DTULTATU,
                ELEMENTO.NRORGINCLUSAO,
                ELEMENTO.CDOPERINCLUSAO,
                OPINC.NMOPERADOR                                                                                     AS NMOPERINCLUSAO,
                ELEMENTO.NRORGULTATU,
                ELEMENTO.CDOPERULTATU,
                OPATU.NMOPERADOR                                                                                     AS NMOPERULTATU,
                DECODE(ELEMENTO.IDELEMENTOPAI, NULL, 0, ELEMENTO.IDORDENAELEMENTO)                                   AS IDORDENAELEMENTO_TELA,
                ELEMENTO.IDORDENAELEMENTO,
                ELEMENTO.CDTPMEDIDAVARIAVEL,
                MEDIDA.DSTPMEDIDASERV,
                MEDIDA.CDTPMEDIDASERV,
                DECODE(NVL(ELEMENTO.IDELEMENTO, 'G_' || ELEMENTO.CDSERVORCA), 'G_' || ELEMENTO.CDSERVORCA, 'N', 'S') AS IDBLOQUEADO,
                ELEMENTO.IDFORMACAOPRECO,
                ELEMENTO.CDCENTCUST,
                CUSTO.NMCENTCUST, 
                DECODE((SELECT COUNT(*) AS QTFILHOS
				            FROM FCT_SERVICOORCA SERV 
						   WHERE SERV.IDELEMENTOPAI = ELEMENTO.IDELEMENTO), 0, 0, 1) AS HASCHILD
            FROM
                FCT_SERVICOORCA    ELEMENTO
                LEFT JOIN FCT_SERVICOORCA    PAI ON ELEMENTO.IDELEMENTOPAI = PAI.IDELEMENTO
                                                AND ELEMENTO.NRORG = PAI.NRORG
                LEFT JOIN FCT_TIPOMEDIDASERV MEDIDA ON ELEMENTO.CDTPMEDIDAVARIAVEL = MEDIDA.CDTPMEDIDASERV
                                                    AND ELEMENTO.NRORG = MEDIDA.NRORG
                LEFT JOIN OPERADOR           OPINC ON ELEMENTO.CDOPERINCLUSAO = OPINC.CDOPERADOR
                                            AND ELEMENTO.NRORG = OPINC.NRORG
                LEFT JOIN OPERADOR           OPATU ON ELEMENTO.CDOPERULTATU = OPATU.CDOPERADOR
                                            AND ELEMENTO.NRORG = OPATU.NRORG
                LEFT JOIN CENTCUST           CUSTO ON ELEMENTO.CDCENTCUST = CUSTO.CDCENTCUST
                                            AND ELEMENTO.NRORG = CUSTO.NRORG	  
               $filtroRelacionamentoServico
            WHERE
                ELEMENTO.IDTIPO = 'E'
            $fetchOnDemand
            $fetchOnDemandPrimeiraLinha
            $filtroIdAtivo
            $filtroNome
            $filtroTipoMedidaVariavel
            $filtroFormacaoPreco
            $filtroCentroCusto
            $idTipoElemento
            ORDER BY NMSERVORCA ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTipoMedidaVariavel()
    {

        $sql = "SELECT
                    CDTPMEDIDASERV, 
                    DSTPMEDIDASERV
                FROM
                    FCT_TIPOMEDIDASERV
                WHERE
                        IDATIVO = 'S'
                    AND IDTPMEDIDASERV = 'V'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getRelacionamentoServico()
    {

        $sql = "SELECT DISTINCT CDSERVORCA, NMSERVORCA
                FROM FCT_SERVICOORCA
                WHERE IDATIVO = 'S' AND IDTIPO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getCentroCusto()
    {

        $sql = "SELECT DISTINCT
                CENTCUST.CDCENTCUST,
                CENTCUST.NMCENTCUST,
                CENTCUST.NRORG
                FROM CENTCUST
                JOIN CENTCUSTFIL ON CENTCUSTFIL.CDCENTCUST = CENTCUST.CDCENTCUST
                AND CENTCUSTFIL.NRORG = CENTCUST.NRORG";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function frameRelacionamentoServico($parametros)
    {

        $sql = "SELECT
                FCT_RELACSERVICO.IDMEDIDAFIXA,
                FCT_RELACSERVICO.NRRELACSERVICO,
                FCT_RELACSERVICO.CDSERVORCA,
                FCT_RELACSERVICO.CDSERVORCARELAC,
                FCT_SERVICOORCA.NMSERVORCA AS NMSERVORCARELAC,
                FCT_RELACSERVICO.CDTPMEDIDASERV,
                FCT_TIPOMEDIDASERV.DSTPMEDIDASERV,
                FCT_RELACSERVICO.QTMEDIDASERV,
                FCT_RELACSERVICO.NRORG,
                FCT_RELACSERVICO.DTINCLUSAO,
                FCT_RELACSERVICO.DTULTATU,
                FCT_RELACSERVICO.NRORGINCLUSAO,
                FCT_RELACSERVICO.CDOPERINCLUSAO,
                OPINC.NMOPERADOR           AS NMOPERINCLUSAO,
                FCT_RELACSERVICO.NRORGULTATU,
                FCT_RELACSERVICO.CDOPERULTATU,
                OPATU.NMOPERADOR           AS NMOPERULTATU,
                FCT_RELACSERVICO.IDATIVO
                FROM FCT_RELACSERVICO
                INNER JOIN FCT_SERVICOORCA ON FCT_RELACSERVICO.CDSERVORCARELAC = FCT_SERVICOORCA.CDSERVORCA
                                            AND FCT_RELACSERVICO.NRORG = FCT_SERVICOORCA.NRORG
                                            AND FCT_SERVICOORCA.IDTIPO = 'S'                                            
                LEFT JOIN FCT_TIPOMEDIDASERV ON FCT_RELACSERVICO.CDTPMEDIDASERV = FCT_TIPOMEDIDASERV.CDTPMEDIDASERV
                                            AND FCT_RELACSERVICO.NRORG = FCT_TIPOMEDIDASERV.NRORG
                LEFT JOIN OPERADOR OPINC ON FCT_RELACSERVICO.CDOPERINCLUSAO = OPINC.CDOPERADOR
                                            AND FCT_RELACSERVICO.NRORG = OPINC.NRORG
                LEFT JOIN OPERADOR OPATU ON FCT_RELACSERVICO.CDOPERULTATU = OPATU.CDOPERADOR
                                            AND FCT_RELACSERVICO.NRORG = OPATU.NRORG
                WHERE FCT_RELACSERVICO.CDSERVORCA = :CDSERVORCA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function salvarElementoAdd($parametros)
    {

        $sql = "INSERT INTO FCT_SERVICOORCA 
                       (IDATIVO,
                        NMSERVORCA,
                        IDELEMENTOPAI,
                        CDTPMEDIDAVARIAVEL,
                        IDORDENAELEMENTO,
                        IDFORMACAOPRECO,
                        IDALTERAFORMPREC,
                        DSSERVORCA,
                        CDCENTCUST,
                        NRORG,
                        CDOPERINCLUSAO,
                        CDSERVORCA,
                        IDELEMENTO,
                        IDTIPO,
                        IDITEMCALCULO,
                        IDTIPOCALCULO,
                        DSEXPRESSAOCALC,
                        VRPERCCALCULO,
                        DTINCLUSAO)

                VALUES (:IDATIVO,
                        :NMSERVORCA,
                        :IDELEMENTOPAI,
                        :CDTPMEDIDAVARIAVEL,
                        :IDORDENAELEMENTO,
                        :IDFORMACAOPRECO,
                        :IDALTERAFORMPREC,
                        :DSSERVORCA,
                        :CDCENTCUST,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :CDSERVORCA,
                        :IDELEMENTO,
                        'E',
                        :IDITEMCALCULO,
                        :IDTIPOCALCULO,
                        :DSEXPRESSAOCALC,
                        :VRPERCCALCULO,
                        SYSDATE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function salvarElementoEdit($parametros)
    {

        $sql = "UPDATE FCT_SERVICOORCA
                SET IDATIVO            = :IDATIVO,
                    NMSERVORCA         = :NMSERVORCA,
                    IDELEMENTOPAI      = :IDELEMENTOPAI,
                    CDTPMEDIDAVARIAVEL = :CDTPMEDIDAVARIAVEL,
                    IDORDENAELEMENTO   = :IDORDENAELEMENTO,
                    IDFORMACAOPRECO    = :IDFORMACAOPRECO,
                    IDALTERAFORMPREC   = :IDALTERAFORMPREC,
                    DSSERVORCA         = :DSSERVORCA,
                    CDCENTCUST         = :CDCENTCUST,
                    CDOPERULTATU       = :CDOPERULTATU,
                    NRORGULTATU        = :NRORGULTATU,
                    IDITEMCALCULO      = :IDITEMCALCULO,
                    IDTIPOCALCULO      = :IDTIPOCALCULO,
                    DSEXPRESSAOCALC    = :DSEXPRESSAOCALC,
                    VRPERCCALCULO      = :VRPERCCALCULO,
                    DTULTATU           = SYSDATE
                WHERE CDSERVORCA = :CDSERVORCA";

        $this->connection->executeStatement($sql, $parametros);

        $sql = "UPDATE FCT_SERVICOORCA
                SET IDATIVO            = :IDATIVO,
                    CDOPERULTATU       = :CDOPERULTATU,
                    NRORGULTATU        = :NRORGULTATU,
                    DTULTATU           = SYSDATE
                WHERE IDELEMENTOPAI = :IDELEMENTOPAI2";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function excluirElementos($parametros)
    {

        $sql = "DELETE FROM FCT_SERVICOORCA
                WHERE CDSERVORCA IN (".$parametros.")";
        
        $this->connection->executeStatement($sql);
    }

    public function salvarRelacionamentoServicoEdit($params)
    {
        $sql = "UPDATE FCT_RELACSERVICO
                   SET IDMEDIDAFIXA     = :IDMEDIDAFIXA, 
                       QTMEDIDASERV     = :QTMEDIDASERV,
                       NRORGULTATU      = :NRORGULTATU,
                       CDOPERULTATU     = :CDOPERULTATU,
                       DTULTATU         = SYSDATE
                WHERE CDSERVORCA   = :CDSERVORCA
                AND NRRELACSERVICO = :NRRELACSERVICO";

        $this->connection->executeStatement($sql, $params);
    }

    public function getTipoMedida()
    {

        $sql = "SELECT CDTPMEDIDASERV, DSTPMEDIDASERV
                FROM FCT_TIPOMEDIDASERV
                WHERE IDATIVO = 'S'
                    AND IDTPMEDIDASERV = 'F'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function salvarRelacionamentoServicoAdd($parametros)
    {

        $sql = "INSERT INTO FCT_RELACSERVICO 
                       (NRRELACSERVICO,
                        CDSERVORCA,
                        CDSERVORCARELAC,
                        QTMEDIDASERV,
                        CDTPMEDIDASERV,
                        IDMEDIDAFIXA,
                        NRORG,
                        CDOPERINCLUSAO,
                        DTINCLUSAO)

                VALUES (:NRRELACSERVICO,
                        :CDSERVORCA,
                        :CDSERVORCARELAC,
                        :QTMEDIDASERV,
                        :CDTPMEDIDASERV,
                        :IDMEDIDAFIXA,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $parametros);
    }

    public function deletarRelacionamentoServico($parametros)
    {

        $sql = "DELETE FROM FCT_RELACSERVICO
                WHERE NRRELACSERVICO IN (".$parametros.")";

        $this->connection->executeStatement($sql);
    }

    public function salvarCadastramentoAutomatico($parametros)
    {

        $sql = "INSERT INTO FCT_RELACSERVICO 
                       (NRRELACSERVICO,
                        CDSERVORCA,
                        CDSERVORCARELAC,
                        QTMEDIDASERV,
                        CDTPMEDIDASERV,
                        NRORG,
                        CDOPERINCLUSAO,
                        DTINCLUSAO)

                VALUES (:NRRELACSERVICO,
                        :CDSERVORCA,
                        :CDSERVORCARELAC,
                        :QTMEDIDASERV,
                        :CDTPMEDIDASERV,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $parametros);
    }
}
