<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;

class SmNextDocumentoCdaFactory
{
    protected $connection;
    
    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * Exclui os documentos temporários a ser utilizado para gerar o relatório.
     */
    public function excluirDocumentosTemporarios(array $params)
    {
        $sql = "DELETE FROM FCT_TMP_DOCUMENTOORCA
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna o documento da proposta para o escopo informado.
     */
    public function getDocumentoCDA($parametros)
    {
        $sql = "SELECT * FROM MODELODOCCDAIDI
                    JOIN SERVPRESTDOC
                    ON MODELODOCCDAIDI.NRMODELODOCCDA = SERVPRESTDOC.NRMODELODOCCDA
                    WHERE SERVPRESTDOC.CDEMPRESA = :CDEMPRESA
                                    AND SERVPRESTDOC.CDFUNC = :CDFUNC
                                    AND SERVPRESTDOC.CDCLIENTE = :CDCLIENTE
                                    AND SERVPRESTDOC.DTSERVPREST = :DTSERVPREST
                                    AND SERVPRESTDOC.NRSERVPREST = :NRSERVPREST";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }
    
     /**
     * Retorna o representante do cliente da CDA.
     */
    public function getDadosRepresentanteCliCDA($parametros)
    {
        $sql = "SELECT
                    INTECLIE.CDINTECLIE,
                    INTECLIE.NMINTECLIE,
                    INTECLIE.NRORG,
                    INTECLIE.CDCLIENTE,
                    INTECLIE.DSEMAILICLIE,
                    INTECLIE.NRCPFICLIE,
                    INTECLIE.NRFAXINCLIE,
                    INTECLIE.NRTEL1INCLIE
                FROM
                    INTECLIE
                WHERE
                    CDCLIENTE = :CDCLIENTE
                    AND CDINTECLIE =:CDINTECLIE";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    /**
     * Retorna os dados principais da proposta.
     */
    public function getDadosCDA($parametros)
    {
        $sql = "SELECT
                        SERVPREST.CDEMPRESA,
                        SERVPREST.CDFUNC,
                        SERVPREST.CDCLIENTE,
                        SERVPREST.CDINTECLIE,
                        CLIENTE.NMRAZSOCCLIE,
                        CLIENTE.NMFANTCLIE,
                        SERVPREST.NRLICENCAUSO,
                        LICENCAUSO.NMFANTLUSO,
                        TO_CHAR(SERVPREST.DTSERVPREST, 'DD/MM/YYYY') DTSERVPREST,
                        SERVPREST.NRPROPOSFAS || '-' || SERVPREST.NRVERPROPFAS AS PROPOSTA,
                        SERVPREST.CDTIPOSERV,
                        TIPOSERV.NMTIPOSERV,
                        SERVPREST.NMINTCLIENTE,
                        SERVPREST.QTHRTOTAL,
                        SERVPREST.QTHRATIVIDADEFAT,
                        SERVPREST.QTHRDESLOCAMENTOFAT,
                        TO_CHAR(SERVPREST.DTENTRADA, 'HH24:MI') DTENTRADA,
                        TO_CHAR(SERVPREST.DTSAIDA, 'HH24:MI') DTSAIDA,
                        SERVPREST.HRAUSENCIAALMOCO,
                        SERVPREST.CDPROJETO,
                        SERVPREST.NRSERVPREST,
                        PROJETO.NMPROJETO,
                        DESCSERVPRES.DSSERVPREST,
                        FUNCIONARIO.NMFUNC
                    FROM SERVPREST
                    JOIN CLIENTE 
                    ON CLIENTE.CDCLIENTE = SERVPREST.CDCLIENTE
                    LEFT JOIN LICENCAUSO 
                    ON LICENCAUSO.NRLICENCAUSO = SERVPREST.NRLICENCAUSO
                    AND LICENCAUSO.CDCLIENTE = SERVPREST.CDCLIENTE
                    LEFT JOIN PROJETO 
                    ON PROJETO.CDPROJETO = SERVPREST.CDPROJETO
                    JOIN TIPOSERV 
                    ON TIPOSERV.CDTIPOSERV = SERVPREST.CDTIPOSERV
                    JOIN DESCSERVPRES 
                    ON DESCSERVPRES.CDEMPRESA = SERVPREST.CDEMPRESA
                    AND DESCSERVPRES.CDFUNC = SERVPREST.CDFUNC
                    AND DESCSERVPRES.CDCLIENTE = SERVPREST.CDCLIENTE
                    AND DESCSERVPRES.DTSERVPREST = SERVPREST.DTSERVPREST
                    AND DESCSERVPRES.NRSERVPREST = SERVPREST.NRSERVPREST
                    JOIN FUNCIONARIO
                    ON FUNCIONARIO.CDFUNC = SERVPREST.CDFUNC
                    WHERE SERVPREST.CDEMPRESA = :CDEMPRESA
                    AND SERVPREST.CDFUNC = :CDFUNC
                    AND SERVPREST.CDCLIENTE = :CDCLIENTE
                    AND SERVPREST.DTSERVPREST = :DTSERVPREST
                    AND SERVPREST.NRSERVPREST = :NRSERVPREST";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getParticipantesCda($parametros)
    {
        $sql = "SELECT DISTINCT ROWNUM AS IND, DSNOMEPARTICIP, DSSETORPARTICIP
                    FROM SERVPRESTPARTICIP
                    WHERE CDEMPRESA = :CDEMPRESA
                    AND CDFUNC = :CDFUNC
                    AND CDCLIENTE = :CDCLIENTE
                    AND DTSERVPREST = TO_DATE(:DTSERVPREST, 'DD/MM/YYYY')
                    AND NRSERVPREST = :NRSERVPREST";
                
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getHorasNaoFaturadas($parametros)
    {
        $sql = "SELECT JUSTHORASNAOFATUCDA.NRJUSTHORASNAOFATUCDA, JUSTHORASNAOFATUCDA.DSJUSTHORASNAOFATUCDA, 
                    JUSTHORASNAOFATUCDA.TAGJUSTHORASNAOFATUCDA, SERVPRESTJUSTHRNFAT.QTHORASNAOFATU, SERVPRESTJUSTHRNFAT.NRSERVPRESTJUSTHRNFAT
                    FROM JUSTHORASNAOFATUCDA
                    LEFT JOIN SERVPRESTJUSTHRNFAT
                    ON SERVPRESTJUSTHRNFAT.NRJUSTHORASNAOFATUCDA = JUSTHORASNAOFATUCDA.NRJUSTHORASNAOFATUCDA
                    WHERE SERVPRESTJUSTHRNFAT.CDEMPRESA = :CDEMPRESA
                    AND SERVPRESTJUSTHRNFAT.CDFUNC = :CDFUNC
                    AND SERVPRESTJUSTHRNFAT.CDCLIENTE = :CDCLIENTE
                    AND SERVPRESTJUSTHRNFAT.DTSERVPREST = TO_DATE(:DTSERVPREST)
                    AND SERVPRESTJUSTHRNFAT.NRSERVPREST = :NRSERVPREST";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getCodigoItensParticipantes($parametros)
    {
        $sql = "SELECT DISTINCT ROWNUM AS IND,
                       PARTICIP.*
                  FROM (SELECT DSNOMEPARTICIP, DSSETORPARTICIP
                    FROM SERVPRESTPARTICIP
                    WHERE CDEMPRESA = :CDEMPRESA
                    AND CDFUNC = :CDFUNC
                    AND CDCLIENTE = :CDCLIENTE
                    AND DTSERVPREST = TO_DATE(:DTSERVPREST, 'DD/MM/YYYY')
                    AND NRSERVPREST = :NRSERVPREST
                    AND NRORG = :NRORG) PARTICIP";
                   
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }
}
